# coding=utf8

"""A wrapper for the current iteration of the Monzo API.

Handles tasks such as prompting the user to go through the web oauth flow,
capturing the code from Monzo with a built in web server, exchanging the code
for an oauth token and re-issuing it as and when required.

  Typical usage example:

  api = MonzoAPI(client_id, client_secret)
  api.initialize()
  print api.list_accounts()
"""

import BaseHTTPServer
import datetime
import os
import pickle
import urllib
import urlparse

import iso8601
import requests
import rfc3339


class MonzoAPI:
    """Wraps authenticating, calling and de-marshaling Monzo API calls."""

    def __init__(self, client_id, client_secret):
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = 'http://localhost:1234/'
        self.config_file = os.path.join(os.path.expanduser('~'), '.mondofs')
        self.oauth = None

    def initialize(self):
        """Attempt to initialize this instance. We attempt to read an oauth
        token from a config file on disk. If that token exists but is expired
        we attempt to refresh it. If the token is still not valid or present we
        take the user back through the oauth flow to get a new token.
        """

        if os.path.exists(self.config_file):
            # Attempt to reload config from the given config file.
            with open(self.config_file, 'r') as fp:
                self.oauth = pickle.load(fp)

        if self.oauth and self._oauth_expired():
            # If the token was read from the file then lets try and refresh it.
            self._refresh_oauth_token()

        if not self.oauth or self._oauth_expired():
            # If that didn't work we need to ask the user to re-authorize the
            # app.
            self._authorize()

    def _authorize(self):
        """Perform the initial oauth dance by redirecting the user to Monzo. To
        get an auth code that we can exchange for an oauth token.
        """

        # This should be good enough.. We fetch 40 random bytes and map them to
        # characters in the range a-z.
        # TODO(tomhennigan) It's unclear from the docs how secure this token
        # should actually be. Should clarify this and harden if required.
        az = map(chr, range(ord('a'), ord('z') + 1))
        expected_state = ''.join(map(lambda d: az[ord(d) % len(az)],
                                     os.urandom(40)))

        url = 'https://auth.getmondo.co.uk/?'
        url += urllib.urlencode({
            'client_id': self.client_id,
            'redirect_uri': self.redirect_uri,
            'state': expected_state,
            'response_type': 'code',
        })
        print 'Please go to the URL below to authorize this app.'
        print
        print '\033[93m' + url + '\033[00m'
        print
        print 'You\'ll need to give Monzo your email, and they\'ll send you a'
        print 'confirmation email that you can use to sign in to this app.'

        # Start an HTTP server to wait for the callback from Monzo. Once the
        # user has done the dance of filling in their email address, finding
        # the verification email and clicking "Authorize" in that we'll get a
        # callback.
        httpd = BaseHTTPServer.HTTPServer(('localhost', 1234), HTTPServer)
        httpd.handle_request()
        code = HTTPServer.params.get('code', [None])[0]
        state_back = HTTPServer.params.get('state', [None])[0]

        if not code or not state_back:
            raise Exception('Invalid response from callback.')

        if expected_state != state_back:
            raise Exception('State token %s was not OK %s.' % (state_back,
                                                               expected_state))

        # We need to convert the auth code we're given to an oauth token that
        # we can use to authenticate API requests.
        self._fetch_oauth_token(code=code,
                                redirect_uri=self.redirect_uri,
                                grant_type='authorization_code')

    def _oauth_expired(self):
        """Tests whether the oauth token has expired."""
        return datetime.datetime.now() > self.oauth['_expires']

    def _fetch_oauth_token(self, **params):
        """Fetches an oauth token. Requests should set keyword arguments for
        additional POST parameters to the /oauth2/token endpoint.
        """

        # Exchange the code from the callback for an oauth token.
        params['client_id'] = self.client_id
        params['client_secret'] = self.client_secret
        r = requests.post('https://api.getmondo.co.uk/oauth2/token',
                          data=params)

        self.oauth = r.json()
        now = datetime.datetime.now()
        expires = self.oauth.get('expires_in', -1)
        self.oauth['_expires'] = now + datetime.timedelta(milliseconds=expires)

        with open(self.config_file, 'w') as fp:
            pickle.dump(self.oauth, fp)

    def _refresh_oauth_token(self):
        """Forces a refresh of the oauth token."""
        self._fetch_oauth_token(grant_type='refresh_token',
                                refresh_token=self.oauth['refresh_token'])

    def _get_access_token(self):
        """Gets a valid and non-expired oauth token."""
        if self._oauth_expired():
            self._refresh_oauth_token()

        return self.oauth.get('access_token', None)

    def _get(self, path, params=None):
        """Executes a GET request to the Monzo API.

        :param params: An optional dictionary of parameters.
        :returns: The de-marshaled response from the API (e.g. a dict).
        """
        url = 'https://api.getmondo.co.uk/' + path
        if params:
            url += ('?' + urllib.urlencode(params))
        headers = {
            'Authorization': 'Bearer ' + self._get_access_token(),
        }
        return requests.get(url, headers=headers).json()

    def get_accounts(self):
        """https://getmondo.co.uk/docs/#accounts"""
        return self._get('accounts').get('accounts', [])

    def get_balance(self, account_id):
        """https://getmondo.co.uk/docs/#balance"""
        return self._get('balance', params={'account_id': account_id})

    def list_transactions(self, account_id, date_from, date_to):
        """https://getmondo.co.uk/docs/#list-transactions"""
        return list(self._list_transactions(account_id, date_from, date_to))

    def _list_transactions(self, account_id, date_from, date_to):
        """Fetch all transactions within the given date ranges. Handles
        pagination.

        :returns: A generator that yields all transactions within the range.
        """
        start = date_from
        limit = 100
        while True:
            since = rfc3339.rfc3339(start,
                                    use_system_timezone=False,
                                    utc=True)
            before = rfc3339.rfc3339(date_to,
                                     use_system_timezone=False,
                                     utc=True)
            params = {
                'account_id': account_id,
                'limit': limit,
                'since': since,
                'before': before,
            }
            response = self._get('transactions', params=params)
            transactions = response.get('transactions', [])

            last_created = None
            for transaction in transactions:
                created = iso8601.parse_date(transaction['created'])
                if last_created is None or created > last_created:
                    last_created = created
                yield transaction

            if len(transactions) < limit:
                # No need to paginate.
                return

            start = last_created

    def get_transaction(self, transaction_id, merchant):
        """https://getmondo.co.uk/docs/#retrieve-transaction"""
        params = {}
        if merchant:
            params['expand[]'] = 'merchant'
        result = self._get('transactions/' + transaction_id, params=params)
        return result.get('transaction', {})


class HTTPServer(BaseHTTPServer.BaseHTTPRequestHandler):
    """An HTTP server capable of handling a GET request."""

    def do_GET(self):
        if self.path.startswith('/?'):
            HTTPServer.params = urlparse.parse_qs(self.path[2:])
        else:
            HTTPServer.params = {}

        self.send_response(302)
        self.send_header('Content-type', 'text/html')
        self.end_headers()

        w = self.wfile.write
        w('<!DOCTYPE html>')
        w('<html>')
        w('  <head>')
        w('    <title>mondo-fs succesfully setup</title>')
        w('    <link rel="stylesheet" href="https://fonts.googleapis.com/icon?family=Material+Icons">')
        w('    <link rel="stylesheet" href="https://code.getmdl.io/1.2.0/material.indigo-pink.min.css">')
        w('    <script defer src="https://code.getmdl.io/1.2.0/material.min.js"></script>')
        w('    <style>')
        w('      html, body {')
        w('      }')
        w('      body {')
        w('      display: flex;')
        w('      align-items: center;')
        w('      justify-content: center;')
        w('      }')
        w('      .mondo-card {')
        w('      flex: 0 0 300px;')
        w('      height: 300px;')
        w('      }')
        w('      .mondo-card > .mdl-card__title {')
        w('      color: #111;')
        w('      height: 176px;')
        w('      background: #f0f0f0 url(\'http://i.imgur.com/e0rv3HQ.png\') center center no-repeat;')
        w('      }')
        w('    </style>')
        w('  </head>')
        w('  <body>')
        w('    <div class="mondo-card mdl-card mdl-shadow--2dp">')
        w('      <div class="mdl-card__title">')
        w('        <h2 class="mdl-card__title-text">Success!</h2>')
        w('      </div>')
        w('      <div class="mdl-card__supporting-text">')
        w('        You\'ve successfully setup monzo-fs. Please navigate to the mount point in your favorite file browser to try it out!')
        w('      </div>')
        w('      <div class="mdl-card__actions mdl-card--border">')
        w('        <a href="https://github.com/tomhennigan/monzo-fs" class="mdl-button mdl-button--colored mdl-js-button mdl-js-ripple-effect" data-upgraded=",MaterialButton,MaterialRipple">')
        w('        Learn More')
        w('        <span class="mdl-button__ripple-container"><span class="mdl-ripple"></span></span></a>')
        w('      </div>')
        w('    </div>')
        w('  </body>')
        w('</html>')



# coding=utf8

"""Utility functions and decorators used in monzo-fs."""

import datetime
import json


_singleton = {}


def singleton(key, value=None):
    """Gets or sets a singleton instance.

    :param key: the key used to identify this singleton.
    :param value: (optional) if set this is the value for the singleton.
    :return: singleton value for key.
    """
    global _singleton

    if value is not None:
        if key in _singleton:
            raise Exception('Singleton %s was already set.' % key)
        _singleton[key] = value

    if key not in _singleton:
        raise Exception('Singleton %s has not been set.' % key)

    return _singleton[key]


def cache(timedelta):
    """Returns a decorator that memoizes the wrapped function (based on
    JSON serialization of positional and keyword args).

    :param timedelta: A datetime.timedelta instance for cache lifetime.
    :returns: A callable that can be used as a decorator for a function.
    """

    cache = {}
    cache_expiry = {}

    if type(timedelta) is not datetime.timedelta:
        raise Exception('timedelta argument must have type datetime.timedelta')

    def _decorator(fn):
        def _cache(*args, **kwargs):
            key = json.dumps(args) + json.dumps(kwargs)
            expires = cache_expiry.get(key, None)

            if expires:
                if datetime.datetime.now() < expires:
                    return cache[key]
                else:
                    cache[key] = None
                    cache_expiry[key] = None

            cache[key] = fn(*args, **kwargs)
            cache_expiry[key] = datetime.datetime.now() + timedelta

            return cache[key]
        return _cache
    return _decorator


def appendnewline(fn):
    """Returns a function that calls the given function and appends an ascii
    newline byte to the result (if the result is not a list or dict). The
    result of the function is also interpreted as bytes.

    :param fn: A callable to be wrapped.
    :return: A callable that invokes fn and appends a newline byte.
    """
    def _decorator(*a, **k):
        r = fn(*a, **k)
        if type(r) not in (list, dict):
            r = bytes(r) + b'\n'
        return r
    return _decorator


def to_2dp(fn):
    """Converts pence (e.g. 1000) to 2dp (e.g. 10.00)."""
    return lambda *a, **k: '%.02f' % (float(fn(*a, **k)) / 100.0)



# coding=utf8

"""Defines the various functions to support the Monzo fuse filesystem."""

import calendar
import datetime
import json

import diazed
from monzo_fs.decorators import cache, singleton, appendnewline, to_2dp
from monzo_fs.diazed import readdir, readlink, mixed
from monzo_fs.monzo import MonzoAPI


def transaction_list_cache():
    """Caches the result from listing all transactions in a given month. This
    transaction object only contains partial info (e.g. no merchant details).

    :returns: a singleton dict instance that can be used as a cache.
    """
    try:
        return singleton('transaction-list-cache')
    except:
        return singleton('transaction-list-cache', {})


@cache(datetime.timedelta(minutes=5))
def _get_transaction(transaction_id, merchant):
    """Return a transaction dict for the transaction with the given id.
    Optionally with merchant details.

    :param transaction_id: The transaction id to fetch.
    :param merchant: Whether to fetch merchant details.
    :returns: A transaction object optionally with merchant details.
    """
    # If we're not getting merchant details and we've seen this in the list
    # cache then we can fetch it from there :)
    cache = transaction_list_cache()
    if not merchant and transaction_id in cache:
        return cache[transaction_id]

    return singleton(MonzoAPI).get_transaction(transaction_id, merchant)


@readdir('/')
@cache(datetime.timedelta(days=1))
def list_accounts():
    """List out all the account IDs for the current user."""
    return [a['id'] for a in singleton(MonzoAPI).get_accounts()]


@readdir('/<account>')
@cache(datetime.timedelta(days=1))
def list_account(account_id):
    """For a specific account list the subfolders that are available."""
    return ['transactions', 'balance']


@readdir('/<account>/transactions')
def transactions(account_id):
    """List out the years for which we could have transaction data."""
    return [str(y) for y in xrange(2015, datetime.datetime.now().year + 1)]


@readdir('/<account>/transactions/<year>')
def months_in_year(account_id, year):
    """List out the months for which transaction data could be avaialble."""
    today = datetime.datetime.now()
    if int(year) != today.year:
        return ['%02d' % m for m in xrange(1, 13)]
    else:
        return ['%02d' % m for m in xrange(1, today.month + 1)]


@readdir('/<account>/transactions/<year>/<month>')
@cache(datetime.timedelta(minutes=1))
def transactions_in_year_month(account_id, year, month):
    """List the transaction ids that occurred in the given year/month."""
    year = int(year)
    month = int(month)
    date_from = datetime.datetime(year=year, month=month, day=1)
    date_to = datetime.datetime(year=year,
                                month=month,
                                day=calendar.monthrange(year, month)[1])
    transactions = singleton(MonzoAPI).list_transactions(account_id,
                                                         date_from,
                                                         date_to)
    # Cache the result of listing the transactions so we can re-use it.
    cache = transaction_list_cache()
    for transaction in transactions:
        transaction['merchant'] = {}
        cache[transaction['id']] = transaction

    # Just return the ids which act as folders.
    return [t['id'] for t in transactions]


@readdir('/<account>/transactions/<year>/<month>/<txn>')
def transaction_fields(account_id, year, month, transaction_id):
    """List the fields available in the transaction."""
    return _get_transaction(transaction_id, False).keys() + ['json']


@readlink('/<account>/transactions/<year>/<month>/<txn>/json')
def transaction_as_json(account_id, year, month, transaction_id):
    """A special file to print the given transaction as JSON."""
    return json.dumps(_get_transaction(transaction_id, True))


@mixed(
    operations=[
        'readlink',
        'readdir'
    ],
    paths=[
        '/<account>/transactions/<year>/<month>/<txn>/<f1>',
        '/<account>/transactions/<year>/<month>/<txn>/<f1>/<f2>',
        '/<account>/transactions/<year>/<month>/<txn>/<f1>/<f2>/<f3>'
    ]
)
@appendnewline
def field_from_transaction(account_id, year, month, transaction_id,
                           field, subfield=None, subsubfield=None):
    merchant = (field == 'merchant')
    txn = _get_transaction(transaction_id, merchant)
    if not merchant:
        txn['merchant'] = {}

    ret = txn.get(field, '')
    if subfield:
        ret = ret.get(subfield, '')
    if subsubfield:
        ret = ret.get(subsubfield, '')
    if type(ret) is dict:
        ret = ret.keys()

    if field in ('amount', 'local_amount', 'account_balance'):
        return '%.02f' % (float(ret) / 100.0)

    return ret


@cache(datetime.timedelta(seconds=30))
def _get_balance(account_id):
    return singleton(MonzoAPI).get_balance(account_id)


@readdir('/<account>/balance')
def list_balance(account_id):
    return ['balance', 'currency', 'spend_today']


@readlink('/<account>/balance/balance')
@appendnewline
@to_2dp
def balance_balance(account_id):
    return _get_balance(account_id).get('balance', '')


@readlink('/<account>/balance/currency')
@appendnewline
def balance_currency(account_id):
    return _get_balance(account_id).get('currency', '')


@readlink('/<account>/balance/spend_today')
@appendnewline
@to_2dp
def balance_spend_today(account_id):
    return _get_balance(account_id).get('spend_today', '')



#!/usr/bin/env python
# coding=utf8

import argparse
import logging
import os

import diazed
import fuse
from monzo_fs.decorators import singleton
from monzo_fs.monzo import MonzoAPI


def main():
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('mount_point', help='location to mount the file system')
    parser.add_argument('--logfile', default=None)
    parser.add_argument('--verbose', action='store_true', default=False)
    parser.add_argument('--background', action='store_true', default=False)
    parser.add_argument('--client_id',
                        required=True,
                        help='Your Monzo API client.')
    parser.add_argument('--client_secret',
                        required=True,
                        help='Your Monzo API secret.')
    args = parser.parse_args()

    logging.basicConfig(
        filename=args.logfile,
        level=(logging.DEBUG if args.verbose else logging.INFO))

    m = singleton(MonzoAPI, MonzoAPI(args.client_id, args.client_secret))

    # Perform initialization, which involves authorizing the user if required.
    m.initialize()

    if not os.path.exists(args.mount_point):
        os.mkdir(args.mount_point)

    fuse.FUSE(diazed.fs,
              args.mount_point,
              foreground=(not args.background),
              direct_io=True)

if __name__ == '__main__':
    main()



# coding=utf8

"""Diazed is a bottle inspired framework for writing FUSE file systems.

Diazed refers to D-type fuse cartridges which are commonly known as "bottle
fuses". https://en.wikipedia.org/wiki/IEC_60269#D-system_.28DIAZED.29

The framework works by users annotating functions to handle specific paths.
Paths are converted to regular expressions to match requests coming from the
OS, and groups are defined as <name>. If a path is matched then the groups are
passed through to the function as postional arguments. Rules are matched in
order, so if you had a function decorated with @readlink('/<foo>') and
say @readlink('/<bar>') it would not be possible to reach the second function
because it would be shadowed by the first.

Handler functions can return primative types (e.g. a directory can be
represented as a list of strings) or they can return diazed.Dir or diaezd.File
instances.

  Typical usage example:

  import fuse
  from diazed import fs, readlink, readdir

  @readdir('/')
  def list_root():
      return ['example.txt', 'foo.txt', 'bar.txt']

  @readlink('/example.txt')
  def read_example():
      return 'Hello, world!'

  @readlink('/<file>')
  def read_file(file):
      # This method will not match example.txt because read_example was
      # registered first.
      return 'You are reading %s.' % file

  fuse.FUSE(fs, '/tmp/myfs', foreground=True, direct_io=True)
"""

import collections
import errno
import re
import stat

import fuse


class Dir:
    """Represents a directory containing a list of nodes."""

    def __init__(self, contents, **attrs):
        """Construct a Dir instance.

        :param contents: The iterable contents of the directory.
        :param attrs: Additional attributes for this dir.
        """
        self.contents = list(contents)
        self.attrs = dict(st_mode=(stat.S_IFDIR | 0o555))
        for key, value in attrs.iteritems():
            self.attrs[key] = value

    def __iter__(self):
        return iter(self.contents)


class File:
    """Represents a file with byte contents."""

    def __init__(self, contents):
        """Constructs a File instance.

        :param contents: The contents of the file, which will become bytes.
        """

        self.contents = bytes(contents)
        self.attrs = dict(st_mode=(stat.S_IFREG | 0o444),
                          st_size=len(self.contents))

def _resolve_fs(_fs):
    """Resolves a specific file system, or returns the global one.

    :param _fs: A specific file system to use, or None.
    :returns: An _DiazedFileSystem instance.
    """
    if _fs:
        return _fs
    else:
        global fs
        return fs


def readlink(path, _fs=None):
    """Registers a function that reads a 'file' with the given fs.

    :param path: The path to match (e.g. "/<file>").
    :param _fs: An optional _DiazedFileSystem instance (mostly for testing).
    :returns: A decorator that will register the function with fs for path.
    """
    fs = _resolve_fs(_fs)
    return _get_decorator(fs, operations=['readlink'], paths=[path])


def readdir(path, _fs=None):
    """Decorates a function that lists a 'directory'.

    :param path: The path to match (e.g. "/<file>").
    :param _fs: An optional _DiazedFileSystem instance (mostly for testing).
    :returns: A decorator that will register the function with fs for path.
    """
    fs = _resolve_fs(_fs)
    return _get_decorator(fs, operations=['readdir'], paths=[path])


def mixed(operations, paths, _fs=None):
    """Decorates a function that supports multiple types of action.

    :param operations: The list of operations (e.g. ["readlink"]).
    :param paths: The paths to match (e.g. ["/<file>", "/<file>.txt"]).
    :param _fs: An optional _DiazedFileSystem instance (mostly for testing).
    :returns: A decorator that will register the function with fs for path.
    """
    fs = _resolve_fs(_fs)
    return _get_decorator(fs, operations=operations, paths=paths)


def _get_decorator(fs, operations, paths):
    """Decorator to wrap a function that returns the contents of a path.

    :param paths: the set of paths to handle.
    :return: a File object, bytes or something that will be turned into bytes.
    """
    def _decorator(fn):
        # Register the function as a handler for all the paths + operations.
        for path in paths:
            for operation in operations:
                fs.on(operation, path, _curry(fn, _ensure_obj))
        return fn
    return _decorator


def _ensure_obj(x):
    """Converts 'primative' types to diazed objects (e.g. list -> Dir).

    :param x: The object to cast or pass through.
    :returns: An instance of Dir or File based on the type of x.
    """
    t = type(x)
    if t in (Dir, File):
        return x
    elif t in (list, tuple, set):
        return Dir(x)
    else:
        return File(x)


def _curry(*fns):
    """Returns a function that curries the given functions together.

        >>> a = lambda v: 'a("%s")' % v
        >>> b = lambda v: 'b(%s)' % v
        >>> f = _curry(a, b)
        >>> f("something")
        b(a("something"))

    :param fns: A list of functions to curry.
    :returns: A callable that curries the given functions.
    """
    def __curry(*args, **kwargs):
        ret = fns[0](*args, **kwargs)
        for fn in fns[1:]:
            ret = fn(ret)
        return ret
    return __curry


class _UnableToRouteException(Exception):
    """Thrown when an action is unable to route for the given path."""
    pass


class _DiazedFileSystem(fuse.LoggingMixIn, fuse.Operations):
    """A FUSE file system that forwards syscalls to decorated functions."""

    def __init__(self):
        self.routes = collections.defaultdict(lambda: [])
        self.fd = 0

    def on(self, operation, route, callback):
        """Registers a handler for a specific operation/route pair.

        :param operation: The str name of the operation (e.g. "readlink").
        :param route: The str route to handle (e.g. "/<file>.txt").
        :param callback: A callback to call when route/operation is matched.
        """
        route = '^' + re.sub('<[^>]*>', '([^/]*)', route) + '$'
        self.routes[operation].append((re.compile(route), callback))

    def route(self, operation, path, **fuseargs):
        """Handles a specific routing of a path (e.g. "/foo/bar") to a handler.

        :param operation: The str name of the operation (e.g. "readlink").
        :param path: The str path to handle (e.g. "/foo/bar").
        :param fuseargs: A set of fuse kwargs.
        :throws: _UnableToRouteException if unable to handle path + operation.
        :returns: The value returned by the callback for the given path + op.
        """
        for route, callback in self.routes[operation]:
            match = route.match(path)
            if match is not None:
                # TODO(tomhennigan) Find a way to optionally pass **fuseargs
                #                   without requiring them to be passed. 
                return callback(*match.groups())
        raise _UnableToRouteException('Unable to handle %s' % path)

    def _create_fuse_args(self, **kwargs):
        """Creates a kwargs dict with keys prepended with "_fuse_".

        :param kwargs: The kwargs to copy.
        :returns: A copy of kwargs with keys prefixed with "_fuse_".
        """
        return {'_fuse_' + k: v for k, v in kwargs.iteritems()}

    def _raise_readonlyfs(self):
        raise fuse.FuseOSError(errno.EROFS)

    """File system methods."""

    def readdir(self, path, fh):
        kwargs = self._create_fuse_args(fh=fh)
        return ['.', '..'] + self.route('readdir', path, **kwargs).contents

    def readlink(self, path):
        return self.route('readlink', path).contents

    def getattr(self, path, fh=None):
        kwargs = self._create_fuse_args(fh=fh)

        try:
            return self.route('readdir', path, **kwargs).attrs
        except _UnableToRouteException:
            pass

        try:
            return self.route('readlink', path).attrs
        except _UnableToRouteException:
            pass

        raise fuse.FuseOSError(errno.ENOENT)

    def getxattr(self, path, name, position=0):
        kwargs = self._create_fuse_args(name=name, position=position)
        try:
            return self.route('getxattr', path, **kwargs)
        except _UnableToRouteException:
            pass

        return ''

    def listxattr(self, path):
        try:
            return self.route('listxattr', path)
        except _UnableToRouteException:
            pass

        return []

    def open(self, path, flags):
        kwargs = self._create_fuse_args(flags=flags)
        try:
            self.route('open', path, **kwargs)
        except _UnableToRouteException:
            pass

        self.fd += 1
        return self.fd

    def read(self, path, size, offset, fh):
        kwargs = self._create_fuse_args(size=size, offset=offset, fh=fh)
        try:
            return self.route('read', path, **kwargs).contents
        except _UnableToRouteException:
            pass

        # Fallback to reading the whole file and slicing ourselves.
        return self.readlink(path)[offset:offset + size]

    def statfs(self, path):
        return {}

    def create(self, path, mode):
        self._raise_readonlyfs()

    def write(self, path, data, offset, fh):
        self._raise_readonlyfs()

    def truncate(self, path, length, fh=None):
        self._raise_readonlyfs()

    def unlink(self, path):
        self._raise_readonlyfs()

    def chmod(self, path, mode):
        self._raise_readonlyfs()

    def chown(self, path, uid, gid):
        self._raise_readonlyfs()

    def mkdir(self, path, mode):
        self._raise_readonlyfs()

    def removexattr(self, path, name):
        self._raise_readonlyfs()

    def rename(self, old, new):
        self._raise_readonlyfs()

    def rmdir(self, path):
        self._raise_readonlyfs()

    def setxattr(self, path, name, value, options, position=0):
        self._raise_readonlyfs()

    def symlink(self, target, source):
        self._raise_readonlyfs()

    def utimens(self, path, times=None):
        self._raise_readonlyfs()


# A global file system instance that is used by default.
# TODO(tomhennigan) Should we create this lazily?
fs = _DiazedFileSystem()



#!/usr/bin/env python

import setuptools

setuptools.setup(
     name='monzo-fs',
     version='0.1.3',
     description='A FUSE file system for Mondo bank.',
     author='Tom Hennigan',
     author_email='tomhennigan@gmail.com',
     license='Apache 2.0',
     url='https://github.com/tomhennigan/monzo-fs',
     classifiers=['Development Status :: 3 - Alpha',
                   'Intended Audience :: Developers',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 2.7',
                   ],
    entry_points={
        'console_scripts': [
            'monzo-fs = monzo_fs.__main__:main',
        ],
    },
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=['requests',
                      'fusepy>=2.0.4',
                      'rfc3339',
                      'iso8601'])
